@@ -2,7 +2,7 @@ package Plack::Session::Store::Cache;
 use strict;
 use warnings;
 
-our $VERSION   = '0.09_03';
+our $VERSION   = '0.01';
 our $AUTHORITY = 'cpan:STEVAN';
 
 use Scalar::Util qw[ blessed ];
@@ -24,20 +24,43 @@ sub new {
 }
 
 sub fetch {
-    my ($self, $session_id ) = @_;
-    $self->cache->get($session_id);
+    my ($self, $session_id, $key) = @_;
+    my $cache = $self->cache->get($session_id);
+    return unless $cache;
+    return $cache->{ $key };
 }
 
 sub store {
-    my ($self, $session_id, $session) = @_;
-    $self->cache->set($session_id => $session);
+    my ($self, $session_id, $key, $data) = @_;
+    my $cache = $self->cache->get($session_id);
+    if ( !$cache ) {
+        $cache = {$key => $data};
+    }
+    else {
+        $cache->{$key} = $data;
+    }
+    $self->cache->set($session_id => $cache);
 }
 
-sub remove {
+sub delete {
+    my ($self, $session_id, $key) = @_;
+    my $cache = $self->cache->get($session_id);
+    return unless exists $cache->{$key};
+
+    delete $cache->{ $key };
+    $self->cache->set($session_id => $cache);
+}
+
+sub cleanup {
     my ($self, $session_id) = @_;
     $self->cache->remove($session_id);
 }
 
+sub dump_session {
+    my ($self, $session_id) = @_;
+    $self->cache->get( $session_id ) || {};
+}
+
 1;
 
 __END__
