@@ -2,13 +2,17 @@ package Plack::Session::Store::Null;
 use strict;
 use warnings;
 
-our $VERSION   = '0.09_03';
+our $VERSION   = '0.01';
 our $AUTHORITY = 'cpan:STEVAN';
 
 sub new     { bless {} => shift }
 sub fetch   {}
 sub store   {}
-sub remove  {}
+sub delete  {}
+sub cleanup {}
+sub persist {}
+
+sub dump_session { +{} }
 
 1;
 
@@ -57,7 +61,7 @@ Stevan Little E<lt>stevan.little@iinteractive.comE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2009, 2010 Infinity Interactive, Inc.
+Copyright 2009 Infinity Interactive, Inc.
 
 L<http://www.iinteractive.com>
 
